/// <reference types="node" />
import { ChildProcess } from 'child_process';
import { Stream } from 'stream';
export interface ProcessEnv {
    [key: string]: string | undefined;
}
declare type StdioOptions = 'pipe' | 'ignore' | 'inherit' | Array<('pipe' | 'ipc' | 'ignore' | 'inherit' | Stream | number | null | undefined)>;
declare type StdioNull = 'inherit' | 'ignore' | Stream;
declare type StdioPipe = undefined | null | 'pipe';
declare type StdioPipeOrNull = StdioNull | StdioPipe;
export declare type TStdIO = StdioOptions | [StdioPipeOrNull, StdioPipeOrNull, StdioPipeOrNull];
export declare type TextPredicate = (text: string, next: TextPredicate) => boolean;
export declare type ErrorSearch = (text: string, next: ErrorSearch) => string | void | null | false;
export interface ILogFilters {
    logFilter?: TextPredicate | boolean;
    stdOutSearchError?: ErrorSearch;
    stdErrIsError?: TextPredicate | boolean;
}
export interface IRunOptions extends ILogFilters {
    args?: string[];
    env?: ProcessEnv;
    cwd?: string;
    timeout?: number;
    notAutoKill?: boolean;
    stdin?: undefined | null | 'pipe' | 'ipc' | 'ignore' | 'inherit' | Stream;
    shell?: boolean;
    prepareProcess?: (proc: ChildProcess) => void;
    dontSearchErrors?: boolean;
    ignoreProcessExitCode?: boolean;
    dontShowOutputs?: boolean;
    returnOutputs?: boolean;
}
export interface IGlobalConfig extends ILogFilters {
}
export declare const GLOBAL_CONFIG_ENV = "RUN_SCRIPT_CONFIG_n20fy652y5n";
export declare enum RunStatus {
    ERROR = "ERROR",
    RUNNED = "RUNNED",
    SUCCESS = "SUCCESS"
}
export declare type TRunState = {
    status: RunStatus;
    timeStart: number;
    timeEnd: number;
    description: string;
};
export declare type Func<TThis, TArgs extends any[], TValue = void> = (this: TThis, ...args: TArgs) => TValue;
export {};
