"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.withTimeout = exports.singleCall = exports.singleProcess = void 0;
const log_1 = require("./log");
function singleProcess(descriptionOrFunc, func) {
    let locker;
    return async function _singleProcess(...args) {
        await locker;
        locker = log_1.funcLog(descriptionOrFunc, func).call(this, ...args);
        return locker;
    };
}
exports.singleProcess = singleProcess;
function sortObjectKeys(obj) {
    if (obj == null) {
        return obj;
    }
    if (Array.isArray(obj)) {
        return obj.map(sortObjectKeys);
    }
    if (obj instanceof Object) {
        return Object.keys(obj)
            .sort()
            .reduce((a, key) => {
            a[key] = sortObjectKeys(obj[key]);
            return a;
        }, {});
    }
    return obj;
}
function singleCall(descriptionOrFunc, func) {
    const cache = {};
    return function _singleCall(...args) {
        const id = JSON.stringify(sortObjectKeys(args));
        const cacheItem = cache[id];
        if (cacheItem) {
            if (cacheItem.error) {
                throw cacheItem.error;
            }
            return cacheItem.result;
        }
        if (cacheItem === false) {
            throw new Error(`Recursive call of single call func: ${func || descriptionOrFunc}`);
        }
        cache[id] = false;
        try {
            const result = log_1.funcLog(descriptionOrFunc, func).call(this, ...args);
            cache[id] = { result };
            return result;
        }
        catch (error) {
            cache[id] = { error };
            throw error;
        }
    };
}
exports.singleCall = singleCall;
async function withTimeout(description, timeoutMs, func) {
    if (!timeoutMs) {
        return func();
    }
    let timer;
    const timeoutPromise = new Promise((resolve, reject) => {
        timer = setTimeout(() => reject(new Error(`Timeout expired (${timeoutMs}): ${description}`)), timeoutMs);
    });
    try {
        return await Promise.race([
            func(),
            timeoutPromise,
        ]);
    }
    finally {
        if (timer) {
            clearTimeout(timer);
        }
    }
}
exports.withTimeout = withTimeout;
