"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.killAll = exports.addProcess = exports.wasKillAll = void 0;
const run_state_1 = require("./run-state");
const log_1 = require("./log");
const contracts_1 = require("./contracts");
const simple_tree_kill_1 = require("@flemist/simple-tree-kill");
let _wasKillAll;
function wasKillAll() {
    return _wasKillAll;
}
exports.wasKillAll = wasKillAll;
const processList = [];
function addProcess(proc) {
    processList.push(proc);
}
exports.addProcess = addProcess;
process.on('SIGTERM', () => {
    console.log('SIGTERM');
    killAll({ isFailure: true, syncKill: true });
});
process.on('SIGHUP', () => {
    console.log('SIGHUP');
    killAll({ isFailure: true, syncKill: true });
});
process.on('SIGINT', () => {
    console.log('SIGINT');
    killAll({ isFailure: true, syncKill: true });
});
process.on('SIGBREAK', () => {
    console.log('SIGBREAK');
    killAll({ isFailure: true, syncKill: true });
});
process.on('beforeExit', () => {
    // console.log('beforeExit')
    killAll({ isFailure: false, syncKill: false });
});
process.on('exit', () => {
    console.log('exit');
    killAll({ isFailure: false, syncKill: true });
});
// process.on('disconnect', killAll)
process.on('uncaughtException', err => {
    log_1.printError('uncaughtException', err);
    killAll({ isFailure: true, syncKill: false });
});
function killAll({ isFailure, syncKill, }) {
    if (_wasKillAll) {
        return;
    }
    _wasKillAll = true;
    console.log('Terminating...');
    const kill = () => {
        const procs = processList.filter(o => o.pid && !o.killed && o.pid !== process.pid);
        const pids = procs.map(o => o.pid);
        pids.push(process.pid);
        log_1.printRunStates();
        simple_tree_kill_1.treeKill({
            parentsPids: pids,
            ignorePids: [process.pid],
            force: true,
        });
        if (run_state_1.getRunStates().some(o => o.status === contracts_1.RunStatus.ERROR)) {
            isFailure = true;
        }
        run_state_1.clearRunStates();
        if (isFailure) {
            // eslint-disable-next-line no-process-exit
            process.exit(1);
        }
    };
    if (syncKill) {
        kill();
    }
    else {
        setTimeout(kill, 100);
    }
}
exports.killAll = killAll;
