"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable no-shadow */
const assert_1 = __importDefault(require("assert"));
const stream_1 = require("stream");
const lineByLine_1 = require("./lineByLine");
describe('lineByLine', function () {
    this.timeout(20000);
    function delay(milliseconds) {
        return new Promise(o => setTimeout(o, milliseconds));
    }
    it('run', async function () {
        let log = '';
        const stdout = new stream_1.Writable({
            write(chunk, encoding, callback) {
                log += chunk.toString(encoding === 'buffer' ? void 0 : encoding);
            },
        });
        const inoutStream = new stream_1.Transform({
            transform(chunk, encoding, callback) {
                this.push(chunk);
                callback();
            },
        });
        const lines = [];
        lineByLine_1.lineByLine({
            input: inoutStream,
            maxDelay: 100,
            handler: line => {
                lines.push(line);
            }
        });
        assert_1.default.deepStrictEqual(lines, []);
        inoutStream.write('1');
        assert_1.default.deepStrictEqual(lines, []);
        inoutStream.write('\n');
        assert_1.default.deepStrictEqual(lines, ['1\n']);
        lines.length = 0;
        inoutStream.write('\n\n\n');
        assert_1.default.deepStrictEqual(lines, ['\n', '\n', '\n']);
        lines.length = 0;
        inoutStream.write('2\n3');
        assert_1.default.deepStrictEqual(lines, ['2\n']);
        lines.length = 0;
        inoutStream.write('4\n5');
        assert_1.default.deepStrictEqual(lines, ['34\n']);
        lines.length = 0;
        await delay(50);
        assert_1.default.deepStrictEqual(lines, []);
        await delay(300);
        assert_1.default.deepStrictEqual(lines, ['5']);
        lines.length = 0;
        inoutStream.write('678');
        assert_1.default.deepStrictEqual(lines, []);
        inoutStream.end('9');
        await delay(1);
        assert_1.default.deepStrictEqual(lines, ['6789']);
    });
});
