"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.funcLog = exports.printRunStates = exports.printRunState = exports.printError = void 0;
const kleur_1 = __importDefault(require("kleur"));
const contracts_1 = require("./contracts");
const run_state_1 = require("./run-state");
const kill_1 = require("./kill");
function printError(prefix, err) {
    console.error(kleur_1.default.red().bold(`${prefix}: ${err && err.stack || err && err.toString() || err}`));
}
exports.printError = printError;
function printRunState(status, timeStart, timeEnd, description) {
    const message = `${status} (${((timeEnd || Date.now()) - timeStart) / 1000} sec): ${description}`;
    switch (status) {
        case contracts_1.RunStatus.RUNNED:
            console.log(kleur_1.default.blue(message));
            break;
        case contracts_1.RunStatus.SUCCESS:
            console.log(kleur_1.default.cyan(message));
            break;
        case contracts_1.RunStatus.ERROR:
            console.error(kleur_1.default.red(message));
            break;
        default:
            throw new Error(`Unknown status: ${status}`);
    }
}
exports.printRunState = printRunState;
function printRunStates() {
    const runStates = run_state_1.getRunStates();
    for (let i = 0; i < runStates.length; i++) {
        const state = runStates[i];
        printRunState(state.status, state.timeStart, state.timeEnd, state.description);
    }
}
exports.printRunStates = printRunStates;
function funcLog(descriptionOrFunc, func) {
    return async function _funcLog(...args) {
        let description;
        if (typeof descriptionOrFunc === 'string') {
            description = descriptionOrFunc;
        }
        else {
            func = descriptionOrFunc;
            const funcName = func.name || 'anonymous';
            description = `${funcName}(${JSON.stringify(args).replace(/^\[|]$/g, '')})`;
        }
        console.log(kleur_1.default.blue(`RUN: ${description}`));
        const runState = run_state_1.addRunState({
            status: contracts_1.RunStatus.RUNNED,
            timeStart: Date.now(),
            timeEnd: void 0,
            description,
        });
        try {
            const result = await func.call(this, ...args);
            runState.status = contracts_1.RunStatus.SUCCESS;
            runState.timeEnd = Date.now();
            return result;
        }
        catch (err) {
            runState.status = contracts_1.RunStatus.ERROR;
            runState.timeEnd = Date.now();
            if (!kill_1.wasKillAll()) {
                console.error(kleur_1.default.bold().red(`✗ ${description}\r\n${err && err.stack || err && err.toString() || err}`));
            }
            throw err;
        }
    };
}
exports.funcLog = funcLog;
