"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runOnce = exports.run = exports.removeColor = exports.createColorRegexp = exports.getColorPrefix = void 0;
// eslint-disable-next-line max-len
/* eslint-disable no-unused-vars,callback-return,no-process-exit,no-process-env,no-extra-semi,@typescript-eslint/no-extra-semi,multiline-ternary */
const child_process_1 = require("child_process");
const kleur_1 = __importDefault(require("kleur"));
const globalConfig_1 = require("./globalConfig");
const path_1 = __importDefault(require("path"));
const lineByLine_1 = require("./lineByLine");
const helpers_1 = require("./helpers");
const kill_1 = require("./kill");
const log_1 = require("./log");
// region helpers
function escapeRegExp(string) {
    return string.replace(/[.*+?^${}()|[\]\\]/g, '\\$&'); // $& means the whole matched string
}
// region color
function getColorPrefix(colorFunc) {
    const colorText = colorFunc('COLOR');
    return colorText.match(/^(.*)COLOR/s)[1];
}
exports.getColorPrefix = getColorPrefix;
// eslint-disable-next-line @typescript-eslint/no-shadow
function createColorRegexp(colors) {
    return new RegExp(`[^\\r\\n]*(${colors
        .map(getColorPrefix)
        .map(escapeRegExp)
        .join('|')})[^\\r\\n]*`);
}
exports.createColorRegexp = createColorRegexp;
function removeColor(message) {
    // eslint-disable-next-line no-control-regex
    return message.replace(/\u001B\[\d+m/g, '');
}
exports.removeColor = removeColor;
// endregion
// endregion
// region output handlers
// region stdOutSearchError
// const errorTextRegExp = /\b(err(ors?)?|warn(ings?)?|fail(ed|ure|s)?)\b|[✗]/i
const errorTextRegExp = /[^\r\n]*(\b[1-9]\d* *(fail|err)|[✗×]|fatal error|error occur)[^\r\n]*/i;
const errorColorRegExp = createColorRegexp([
    kleur_1.default.bold,
    kleur_1.default.red,
    kleur_1.default.magenta,
    // colors.yellow,
    kleur_1.default.bgRed,
    kleur_1.default.bgMagenta,
    // colors.bgYellow,
]);
function stdOutSearchErrorGlobal(text) {
    const config = globalConfig_1.getGlobalConfig();
    return config.stdOutSearchError
        ? config.stdOutSearchError(text, stdOutSearchErrorDefault)
        : stdOutSearchErrorDefault(text);
}
function stdOutSearchErrorDefault(text) {
    const errorColor = text.match(errorColorRegExp);
    text = removeColor(text);
    if (errorColor
        // at least 10 letters
        && (/(\w\W*){10,}/s).test(text)
        && !(/√/s).test(text)
        // electron-builder
        && !(/[┌│]/s).test(text)
        // sapper: "189 kB client.905ef984.js"
        && !(/\b\d+\s+\w+\s+\S+\.js\b/.test(text) && text.length < 100)) {
        return `ERROR COLOR: ${errorColor[0]}`;
    }
    const errorText = text.match(errorTextRegExp);
    if (errorText) {
        return `ERROR TEXT: ${errorText[0]}`;
    }
    return false;
}
// endregion
// region stdErrIsError
function correctLog(message) {
    message = message.replace(/^\s{20,}/, '');
    return message;
}
function stdErrIsErrorGlobal(text) {
    const config = globalConfig_1.getGlobalConfig();
    return typeof config.stdErrIsError === 'function' ? config.stdErrIsError(text, stdErrIsErrorDefault)
        : config.stdErrIsError == null ? stdErrIsErrorDefault(text)
            : config.stdErrIsError || null;
}
function stdErrIsErrorDefault(text) {
    text = removeColor(text);
    if (text.length < 20) {
        return false;
    }
    if (/openssl config failed/.test(text)) {
        return false;
    }
    // web storm
    if (/Debugger attached|Debugger listening on|Waiting for the debugger|nodejs.*inspector/.test(text)) {
        return false;
    }
    // rollup
    if (/treating it as an external dependency|\bcreated\b.*\.js in \d|\bFinished in\b/.test(text)) {
        return false;
    }
    if (text.indexOf('→') >= 0) {
        return false;
    }
    // someone package is outdated
    if (/\bnpm update\b/.test(text)) {
        return false;
    }
    // terminate process
    if (/^\^[A-Z]$/.test(text)) {
        return false;
    }
    // experimental warnings
    if (/ExperimentalWarning: Conditional exports is an experimental feature. This feature could change at any time/.test(text)) {
        return false;
    }
    // Entry module "rollup.config.js" is implicitly using "default" export mode,
    // which means for CommonJS outputthat its default export is assigned to "module.exports".
    // For many tools, such CommonJS output will not be interchangeable with the original ES module.
    // If this is intended, explicitly set "output.exports" to either "auto" or "default",
    // otherwise you might want to consider changing the signature of "rollup.config.js"
    // to use named exports only.
    if (/explicitly set "output.exports" to either "auto" or "default"/.test(text)) {
        return false;
    }
    return true;
}
// endregion
// region logFilter
function logFilterGlobal(text) {
    const config = globalConfig_1.getGlobalConfig();
    return typeof config.logFilter === 'function' ? config.logFilter(text, logFilterDefault)
        : config.logFilter == null ? logFilterDefault(text)
            : config.logFilter || null;
}
function logFilterDefault(text) {
    text = removeColor(text);
    // sapper export
    if (/\s{4,}\S\s[^\w\r\n]*node_modules/.test(text)) {
        return false;
    }
    // Empty space
    if (/^\s*$/s.test(text)) {
        return false;
    }
    return true;
}
function _run(command, { args, env, cwd, timeout, notAutoKill, stdin, shell = true, prepareProcess, dontSearchErrors, ignoreProcessExitCode, dontShowOutputs, returnOutputs, logFilter, stdOutSearchError, stdErrIsError, } = {}) {
    function _logFilter(text) {
        return typeof logFilter === 'function' ? logFilter(text, logFilterGlobal)
            : logFilter == null ? logFilterGlobal(text)
                : logFilter || null;
    }
    function _stdOutSearchError(text) {
        return stdOutSearchError
            ? stdOutSearchError(text, stdOutSearchErrorGlobal)
            : stdOutSearchErrorGlobal(text);
    }
    function _stdErrIsError(text) {
        return typeof stdErrIsError === 'function' ? stdErrIsError(text, stdErrIsErrorGlobal)
            : stdErrIsError == null ? stdErrIsErrorGlobal(text)
                : stdErrIsError || null;
    }
    return Promise.resolve().then(() => {
        const currentDir = process.cwd();
        cwd = path_1.default.resolve(cwd || currentDir);
        let cwdRelative = path_1.default.relative(currentDir, cwd);
        if (cwdRelative.startsWith('..')) {
            cwdRelative = cwd;
        }
        if (cwdRelative === '.') {
            cwdRelative = '';
        }
        let description = '';
        if (cwdRelative) {
            description += cwdRelative + '> ';
        }
        if (timeout) {
            description += '(timeout: ' + Math.round(timeout) + ') ';
        }
        description += command;
        if (kill_1.wasKillAll()) {
            return Promise.reject('Was kill all');
        }
        return log_1.funcLog(description, () => new Promise((resolve, reject) => {
            const proc = child_process_1.spawn(command, args, {
                cwd,
                env: {
                    ...process.env,
                    ...env,
                },
                timeout,
                stdio: [stdin, 'pipe', 'pipe'],
                shell,
            });
            let stdoutString = void 0;
            let stderrString = void 0;
            let stdbothString = void 0;
            const _resolve = () => {
                Promise.all([
                    proc.stdout && new Promise(r => {
                        proc.stdout.on('end', r);
                        if (proc.stdout.readableEnded) {
                            r(void 0);
                        }
                    }),
                    proc.stderr && new Promise(r => {
                        proc.stderr.on('end', r);
                        if (proc.stderr.readableEnded) {
                            r(void 0);
                        }
                    }),
                ])
                    .then(() => {
                    resolve(returnOutputs
                        ? {
                            out: stdoutString,
                            err: stderrString,
                            both: stdbothString,
                        }
                        : void 0);
                });
            };
            const _reject = err => {
                reject(err);
            };
            if (returnOutputs) {
                if (proc.stdout) {
                    stdoutString = '';
                    stdbothString = '';
                    proc.stdout.on('data', chunk => {
                        // const encoding = proc.stdout.readableEncoding
                        const str = chunk.toString(); // encoding === 'buffer' ? void 0 : encoding)
                        stdoutString += str;
                        stdbothString += str;
                    });
                }
                if (proc.stderr) {
                    stderrString = '';
                    stdbothString = '';
                    proc.stderr.on('data', chunk => {
                        // const encoding = proc.stdout.readableEncoding
                        const str = chunk.toString(); // encoding === 'buffer' ? void 0 : encoding)
                        stderrString += str;
                        stdbothString += str;
                    });
                }
            }
            if (!notAutoKill) {
                kill_1.addProcess(proc);
            }
            proc
                .on('disconnect', () => {
                _reject('process.disconnect');
            })
                .on('close', (code, signal) => {
                if (!ignoreProcessExitCode && code) {
                    _reject(`process.close(code=${code}, signal=${signal})`);
                }
                else {
                    _resolve();
                }
            })
                .on('exit', (code, signal) => {
                if (!ignoreProcessExitCode && code) {
                    _reject(`process.exit(code=${code}, signal=${signal})`);
                }
                else {
                    _resolve();
                }
            })
                .on('message', (message) => {
                console.log(`process.message: ${message}`);
            })
                .on('error', err => {
                _reject(err);
            });
            if (proc.stdout) {
                lineByLine_1.lineByLine({
                    input: proc.stdout,
                    maxDelay: 100,
                    handler: line => {
                        try {
                            const lineTrim = line.trim();
                            const error = !dontSearchErrors && _stdOutSearchError(lineTrim);
                            if (!dontShowOutputs && _logFilter(lineTrim)) {
                                line = correctLog(line);
                                process.stdout.write(`${line}`);
                            }
                            if (error) {
                                _reject(`ERROR DETECTED: ${error}`);
                            }
                        }
                        catch (ex) {
                            _reject(ex);
                        }
                    },
                });
            }
            if (proc.stderr) {
                lineByLine_1.lineByLine({
                    input: proc.stderr,
                    maxDelay: 1000,
                    handler: line => {
                        try {
                            const lineTrim = line.trim();
                            const isError = !dontSearchErrors && _stdErrIsError(lineTrim);
                            if (isError) {
                                process.stdout.write(`STDERR: ${line}`);
                                _reject(line);
                                return;
                            }
                            const error = isError == null
                                ? _stdOutSearchError(lineTrim)
                                : null;
                            if (!dontShowOutputs && _logFilter(lineTrim)) {
                                line = correctLog(line);
                                process.stdout.write(`${line}`);
                            }
                            if (error) {
                                _reject(`ERROR DETECTED: ${error}`);
                            }
                        }
                        catch (ex) {
                            _reject(ex);
                        }
                    },
                });
            }
            if (prepareProcess) {
                prepareProcess(proc);
            }
        }))();
    }).catch(err => {
        if (!kill_1.wasKillAll()) {
            return Promise.reject(err);
        }
        return null;
    });
}
function run(command, options = {}) {
    return helpers_1.withTimeout(command, options.timeout, () => _run(command, options));
}
exports.run = run;
Promise.prototype.stopOnError = function stopOnError() {
    return this.catch(err => {
        log_1.printError('Kill on error', err);
        kill_1.killAll({ isFailure: true, syncKill: false });
    });
};
exports.runOnce = helpers_1.singleCall(run);
